<?php

class modSchetmash
{
    /** @var modX $modx */
    public $modx;

    const SERVICE_URL = 'https://online.schetmash.com/lk/api/v1';
    const SERVICE_PORT = 443;

    /**
     * @param modX $modx
     * @param array $config
     */
    function __construct(modX &$modx, array $config = [])
    {
        $this->modx =& $modx;
        $corePath = MODX_CORE_PATH . 'components/modschetmash/';
        $assetsUrl = MODX_ASSETS_URL . 'components/modschetmash/';

        $this->config = array_merge([
            'corePath' => $corePath,
            'modelPath' => $corePath . 'model/',
            'processorsPath' => $corePath . 'processors/',

            'connectorUrl' => $assetsUrl . 'connector.php',
            'assetsUrl' => $assetsUrl,
            'cssUrl' => $assetsUrl . 'css/',
            'jsUrl' => $assetsUrl . 'js/',
        ], $config);

        $this->modx->addPackage('modschetmash', $this->config['modelPath']);
        $this->modx->lexicon->load('modschetmash:default');
    } 
    
    
    public function addCheck($order_id, $operation = 'sell',  $update = false)
    {   
		$report = array('mess' => $this->modx->lexicon('modschetmash_check_err3') , 'status' => false ); 
		$order_id = (int)$order_id;

		$timestamp = new \DateTime();
		$curDate = $timestamp->format('Y-m-d H:i:s');     

        $order = $this->modx->getObject('msOrder', $order_id);
        if(!$order) { return $report; }

		$result = $this->printCheck($order, $operation);		
		
        if($result === false || !is_object($result)) { 
			$result = new \stdClass;
			$result->status = 'no_conn';	
		}   

		if(!$update) {
			$object = $this->modx->newObject('modSchetmashItem');
			$object->set('order_id', $order_id);
			$object->set('data_create', $curDate);			
		} else {
			$object = $this->modx->getObject('modSchetmashItem', $update );
		}

		if(!empty($result->error) ) { $object->set('error', json_encode($result->error));  } 
		if(!empty($result->payload) ) { $object->set('payload', json_encode($result->payload));  } 
		if(!empty($result->id) ) { $object->set('uuid', $result->id); }
		$object->set('sum', $this->formatFloat( $order->get('cart_cost') ) );
		$object->set('type',  $operation  );
		$object->set('status',  $result->status );		

		$report['status']  = $object->save();			
		if($report['status'])	{ $report['mess'] = $this->modx->lexicon('modschetmash_check_add'); }
		
		return $report;	
    }


    private function printCheck($order, $operation)
	{	     
		$checkQuery = new \stdClass;
		$timestamp = new \DateTime( $order->get('createdon') );
		//$timestamp->setTimestamp($data['timestamp']);
        $checkQuery->timestamp = $timestamp->format('d.m.Y H:i:s');
		$checkQuery->external_id = $operation . '_'. MODX_HTTP_HOST . '_'.$order->get('id');
		$checkQuery->service = new \stdClass;

		$api_token = $this->modx->getObject('modSystemSetting', 'modschetmash_api_token' )->get('value');   
		
        if(!empty($api_token)) { 
            $api_token = '?api_token='.$api_token;  
		} else { 
			$api_token = ''; 
		}
		
		$url = $this->modx->getOption('site_url');
		
        $checkQuery->service->callback_url = preg_replace('#\/$#','', $url) . MODX_ASSETS_URL .'components/modschetmash/api.php'.$api_token; 	
			
		$checkQuery->receipt = new \stdClass;
		$checkQuery->receipt->attributes = new \stdClass;
        //$checkQuery->receipt->attributes->sno = $this->modx->getObject('modSystemSetting', 'modschetmash_sno' )->get('value');  			

        $phone = $order->getOne('Address')->get('phone');
        $email = $order->getOption('emailsender');


		if(empty($phone) ||  empty($email)) {  

			$user = $order->getOne('User');	
			$profile = $user->getOne('Profile');

			if(empty($phone)  && $profile) { 
				$phone = $profile->get('phone');
				if(empty($phone)) {
					$phone = $profile->get('mobilephone');
				}
			}

			if(empty($email) && $profile) { 
				$email = $profile->get('email');
			} 
		}	


		if(isset($email) && !empty($email) ) { $checkQuery->receipt->attributes->email = $email; }
		if(isset($phone) && !empty($phone) ) { $checkQuery->receipt->attributes->phone = $phone; }
			
		$checkQuery->receipt->payments = array();
		$checkQuery->receipt->items = array();
		$checkQuery->receipt->total = $this->formatFloat( $order->get('cost') ); //$this->formatFloat( $order->get('cart_cost') );
		

		//foreach ($data['payments'] as $payment)
		//{
			$checkQuery->receipt->payments[] = array(
				'type' => 1, 
				'sum' =>  $this->formatFloat( $order->get('cost') ) //$this->formatFloat( $order->get('cart_cost') )
			);
        //}
        
        $products = $order->getMany('Products');

		foreach ($products as $item)
		{            
			$checkQuery->receipt->items[] = array(
				'name' => $this->formatName( $item->get('name') ),
				'price' => $this->formatFloat( $item->get('price') ),
				'sum' => $this->formatFloat(  $item->get('cost')  ),
				'quantity' => $this->formatFloat( $item->get('count'), 3),
				'tax' => 'none'
                //'tax' => $item->getTaxType(),
                //'tax_sum' =>  $this->formatFloat(  $item->getTaxSum() ),  
			);
        } 

        $payment = $order->getOne('Payment');
        $paymentPrice = $payment->get('price');
        if(!empty($paymentPrice) && $paymentPrice > 0) {
            $paymentPrice = $this->formatFloat( $paymentPrice );
            $checkQuery->receipt->items[] = array(  
                'name' => $this->formatName( $payment->get('name') ), 
				'price' => $paymentPrice,
				'sum' => $paymentPrice,
				'quantity' => 1,
                'tax' => 'none', 
            );
        }

        $delivery = $order->getOne('Delivery');
        $deliveryPrice = $delivery->get('price');

        if(!empty($deliveryPrice) && $deliveryPrice > 0){
            $deliveryPrice = $this->formatFloat( $deliveryPrice );
            $checkQuery->receipt->items[] = array(  
                'name' => $this->formatName( $delivery->get('name') ),
				'price' => $deliveryPrice,
				'sum' => $deliveryPrice,
				'quantity' => 1,
                'tax' => 'none', 
            );

		}  
		
		//$this->modx->log(1, print_r($checkQuery, 1), '', '$checkQuery');


		$token = $this->modx->getObject('modSystemSetting', 'modschetmash_token' )->get('value');  
		$shopid = $this->modx->getObject('modSystemSetting', 'modschetmash_shop_id' )->get('value');  


		if(empty($shopid) ) { 
			$this->modx->log(1,  $this->modx->lexicon('modschetmash_shopid_err') , '', 'modSchetmash'); 
			return false;
		}

		$url = $shopid . '/' . $operation . '?token=';

		if (empty($token ))	{
			$token = $this->getToken();
			if (!$token ) {	return false;	}
        }      

		

		$result = $this->send($url.$token, $checkQuery); 	

		if(!is_object($result) ) { return false;  }

		if (isset($result->code) && ($result->code  ==  2 || $result->code == 4) )	{		
			$token = $this->getToken();
			if (!$token)	{	return false;	}


			$result = $this->send($url.$token, $checkQuery);
			if(!is_object($result) ) { return false;  }	
		}


		if (isset($result->id)  )	{
			return $result; 
		} 		

		return false;
    }
    

    private function formatFloat( $int, $count = 2){
        $int = str_replace(',','.',$int); 
		return	round( (float)$int , $count);
	}  
	
	private function formatName($name){    
		$name = htmlspecialchars_decode($name);
		$name = strip_tags($name);
		//$name = str_replace(array('"',':',"'",'{','}','[',']','&quot;'), '', $name);
		$name = preg_replace('|\s+|',' ', $name);
		$name = trim($name);
		$name = mb_substr( $name, 0, 128, 'UTF-8');
		return $name; 
	}
    

    private function send($_path = '', $_data = null)    {		

		//$this->modx->log(3, print_r($_data, 1), '', 'данные для отправки');	//Удалить!!!

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_URL, static::SERVICE_URL.'/'.$_path);
        //curl_setopt($curl, CURLOPT_PORT , static::SERVICE_PORT);         
        if(isset($_data)) { 
			curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
			curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($_data));  
		  }  
        curl_setopt($curl, CURLOPT_HEADER, false);
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true); 
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);

        $out = curl_exec($curl); 
        $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        $err = curl_error($curl);
		curl_close($curl); 


        if (!$code) {  
			$this->modx->log(1, $this->modx->lexicon('modschetmash_server_err1') . print_r($err , 1), '', 'modSchetmash'); 
			return false; 
		}  	

		if ((int)$code !== 200) { 
			$this->modx->log(1,  $this->modx->lexicon('modschetmash_server_err2') . $code , '', 'modSchetmash');
			return false;
		}		

		//$this->modx->log(3, print_r($out, 1), '', 'ответ сервера');	//Удалить!!!
        return json_decode( $out );		
    }
    

    

    public function getStatusCheck($uuid) {

		$report = array('mess' => $this->modx->lexicon('modschetmash_shopid_err') , 'status' => false ); 
        
		$token = $this->modx->getObject('modSystemSetting', 'modschetmash_token' )->get('value');
		$shopid = $this->modx->getObject('modSystemSetting', 'modschetmash_shop_id' )->get('value'); 

		if(empty($shopid) ) {
			$this->modx->log(1,  $this->modx->lexicon('modschetmash_shopid_err') , '', 'modSchetmash'); 
			$report['mess'] = $this->modx->lexicon('modschetmash_shopid_err');
			return $report;
		}

		
		$url = $shopid . '/report/' . $uuid . '?token=';

		if (empty($token ))	{
			$token = $this->getToken();
			if (!$token ) {	
				return $report;
			}
		}

		$result = $this->send($url.$token);		
		

		if(!is_object($result)  ) {
			$report['mess'] = $this->modx->lexicon('modschetmash_server_err3');
			return $report; 
			}

		if (isset($result->code) && ($result->code  ==  2 || $result->code == 4) )	{	
			$token = $this->getToken();
			if (!$token)	{	
				return $report;
			}

			$result = $this->send($url.$token);
			if(!is_object($result) ) { 
				return $report; 
			}	
		}

		if (isset($result->id)  )	{			 
			return $this->updateCheck($result);
		} 

		return $report; 
    }


    private function getToken()
	{

		$data = new \stdClass;
		$data->login = $this->modx->getObject('modSystemSetting', 'modschetmash_login' )->get('value'); 
		$data->password = $this->modx->getObject('modSystemSetting', 'modschetmash_password' )->get('value');  		

		if(empty($data->login) || empty($data->password) ) { 
			return false; 
		}

		$result = $this->send('token',  $data);	
		
		if ( is_object($result) && isset($result->token) && !empty($result->token) )
		{	
			//$this->modx->setOption('schetmash_token', $result->token);
			$setting = $this->modx->getObject('modSystemSetting', 'modschetmash_token');
			$setting->set('value', $result->token);
			$setting->save();

			$cacheRefreshOptions =  array( 'system_settings' => array() );
			$this->modx->cacheManager->refresh($cacheRefreshOptions);

			return $result->token;
		} else {
			$this->modx->log(1, $this->modx->lexicon('modschetmash_token_err1'), '', 'modSchetmash');
		}


		if ( is_object($result) && isset($result->code)  && $result->code == 1 ) { 
			$this->modx->log(1, $this->modx->lexicon('modschetmash_token_err2'), '', 'modSchetmash');
		}
		return false;
	}  

  



	public function updateCheck($result) {	
		$report = array('mess' => $this->modx->lexicon('modschetmash_check_err1'), 'status' => false ); 		
		if(!is_object($result) || !isset($result->id) || !isset($result->status) || $result->status == 'accept' || $result->status == 'processing')   { 
			return $report; 
		} 		

		$object = $this->modx->getObject('modSchetmashItem', array('uuid' => $result->id));

		if(!empty($result->error) ) { $object->set('error', json_encode($result->error));  } 
		if(!empty($result->payload) ) { $object->set('payload', json_encode($result->payload));  } 	
		$object->set('status',  $result->status );

		if(isset($result->timestamp) && !empty($result->timestamp)) { 
			$datetime = new \DateTime($result->timestamp);  
			$object->set('data_print_end', $datetime->format('Y-m-d H:i:s') );	
		}

		$response = $object->save();	
 
		if($response) {
			 $report = array('mess' => $this->modx->lexicon('modschetmash_check_success'), 'status' => true ); 
		} else {
			 $report['mess'] = $this->modx->lexicon('modschetmash_check_err1');
			 $this->modx->log(1, $this->modx->lexicon('modschetmash_check_err1'), '', 'modSchetmash');
		}

		return $report; 
	}


	public function getProcessingCheck(){

		$result = array(); 

		$query = $this->modx->newQuery('modSchetmashItem');
        $query->select( array('uuid','id','type','order_id'));
		$query->where( array('status:IN' => array('processing', 'accept', 'no_conn') ));
		if ($query->prepare() && $query->stmt->execute()) {
            $result = $query->stmt->fetch(PDO::FETCH_ASSOC);	
        }		

		return $result; 		
	}  



	private function ofd1($ob, $type) {

		switch ($type) {
			case 'sell': $n = 1; break;
			case 'sell_refund': $n = 2; break;
			case 'buy': $n = 3; break;
			case 'buy_refund': $n = 4; break;			
			default: $n = '';	break;
		}
	
        $param = array();	

        $datetime = new \DateTime($ob['receipt_datetime']);
        $param['t'] = $datetime->format('Ymd\THi');
		$param['s'] = $ob['total'];
		$param['fn']= $ob['fn_number'];
		$param['i'] = $ob['fiscal_document_number'];
		$param['fp'] = $ob['fiscal_document_attribute'];
		$param['n'] = $n; 

		return 'https://consumer.1-ofd.ru/v1?'.http_build_query($param);	
	}


	public function getLink($order_id, $type = 'sell'){

		$object = $this->modx->getObject('modSchetmashItem', array( 'status' => 'success', 'type' => $type, 'order_id' => $order_id));
		$payload = $object->get('payload');	
		$type = $object->get('type');
		
		$link = '';

		if(!empty($payload)  && !empty($type)) { 
			$link = $this->ofd1($payload, $type);
		}

		return $link;		
    }

}