<?php

class modSchetmashItemGetListProcessor extends modObjectGetListProcessor
{
    public $objectType = 'modSchetmashItem';
    public $classKey = 'modSchetmashItem';
    public $defaultSortField = 'id';
    public $defaultSortDirection = 'DESC';
    //public $permission = 'list';


    /**
     * We do a special check of permissions
     * because our objects is not an instances of modAccessibleObject
     *
     * @return boolean|string
     */
    public function beforeQuery()
    {
        if (!$this->checkPermissions()) {
            return $this->modx->lexicon('access_denied');
        }

        return true;
    }


    /**
     * @param xPDOQuery $c
     *
     * @return xPDOQuery
     */
    /*
    public function prepareQueryBeforeCount(xPDOQuery $c)
    {
        $query = trim($this->getProperty('query'));
        if ($query) {
            $c->where([
                'name:LIKE' => "%{$query}%",
                'OR:description:LIKE' => "%{$query}%",
            ]);
        }

        return $c;
    }*/


    /**
     * @param xPDOObject $object
     *
     * @return array
     */
    public function prepareRow(xPDOObject $object)
    {
        $array = $object->toArray();
        $array['actions'] = [];
        

        if($array['status'] ==  'processing' || $array['status'] == 'accept'){

            $array['actions'][] = [
                'cls' => '',
                'icon' => 'icon icon-refresh action-green',
                'title' => $this->modx->lexicon('modschetmash_item_update'),
                'action' => 'updateItem',
                'button' => true,
                'menu' => true,
            ];

        } elseif($array['status'] == 'no_conn'){

            $array['actions'][] = [
                'cls' => '',
                'icon' => 'icon icon-refresh action-green',
                'title' => $this->modx->lexicon('modschetmash_item_update'),        
                'action' => 'updateaddItem',
                'button' => true,
                'menu' => true,
            ];

        }

        
        if(!empty($array['payload']) ){

            $array['actions'][] = [
                'cls' => '',
                'icon' => 'icon icon-eye action-gray',
                'title' => $this->modx->lexicon('modschetmash_show_check'),        
                'action' => 'showCheck',
                'button' => true,
                'menu' => true,
            ];
        }

        if(!empty($array['error']) ){

            $array['actions'][] = [
                'cls' => '',
                'icon' => 'icon icon-list action-red',
                'title' => $this->modx->lexicon('modschetmash_error_check'),        
                'action' => 'showError',
                'button' => true,
                'menu' => true,
            ];
        }
 
        // Remove
        $array['actions'][] = [
            'cls' => '',
            'icon' => 'icon icon-trash-o action-red',
            'title' => $this->modx->lexicon('modschetmash_item_remove'),
            'multiple' => $this->modx->lexicon('modschetmash_items_remove'),
            'action' => 'removeItem',
            'button' => true,
            'menu' => true,
        ];

        $array['link'] = $this->ofd1($array['payload'], $array['type']);

        return $array;
    }


    private function ofd1($ob, $type) {

        if(empty($ob) || empty($type)) { return ''; }

		switch ($type) {
			case 'sell': $n = 1; break;
			case 'sell_refund': $n = 2; break;
			case 'buy': $n = 3; break;
			case 'buy_refund': $n = 4; break;			
			default: $n = '';	break;
		}
	
        $param = array();	

        $datetime = new \DateTime($ob['receipt_datetime']);
        $param['t'] = $datetime->format('Ymd\THi');
		$param['s'] = $ob['total'];
		$param['fn']= $ob['fn_number'];
		$param['i'] = $ob['fiscal_document_number'];
		$param['fp'] = $ob['fiscal_document_attribute'];
		$param['n'] = $n; 

        $link = 'https://consumer.1-ofd.ru/v1?'.http_build_query($param);	
        return '<a href="'.$link.'" target="_blank">' . $this->modx->lexicon('modschetmash_check') .'</a>';
     
	}

}

return 'modSchetmashItemGetListProcessor';