modSchetmash.grid.Items = function (config) {
    config = config || {};
    if (!config.id) {
        config.id = 'modschetmash-grid-items';
    }
    Ext.applyIf(config, {
        url: modSchetmash.config.connector_url,
        fields: this.getFields(config),
        columns: this.getColumns(config),
        tbar: this.getTopBar(config),
        sm: new Ext.grid.CheckboxSelectionModel(),
        baseParams: {
            action: 'mgr/item/getlist'
        },
        listeners: {
            rowDblClick: function (grid, rowIndex, e) {
                var row = grid.store.getAt(rowIndex);
                this.updateItem(grid, e, row);
            }
        },
        viewConfig: {
            forceFit: true,
            enableRowBody: true,
            autoFill: true,
            showPreview: true,
            scrollOffset: 0,
            getRowClass: function (rec) {
                return !rec.data.active
                    ? 'modschetmash-grid-row-disabled'
                    : '';
            }
        },
        paging: true,
        remoteSort: true,
        autoHeight: true,
    });
    modSchetmash.grid.Items.superclass.constructor.call(this, config);

    // Clear selection on grid refresh
    this.store.on('load', function () {
        if (this._getSelectedIds().length) {
            this.getSelectionModel().clearSelections();
        }
    }, this);
};
Ext.extend(modSchetmash.grid.Items, MODx.grid.Grid, {
    windows: {},

    getMenu: function (grid, rowIndex) {
        var ids = this._getSelectedIds();

        var row = grid.getStore().getAt(rowIndex);
        var menu = modSchetmash.utils.getMenu(row.data['actions'], this, ids);

        this.addContextMenuItem(menu);
    },

    createItem: function (btn, e) {

        var w = MODx.load({
            xtype: 'modschetmash-item-window-create',
            id: Ext.id(),
            listeners: {
                success: {
                    fn: function () {
                        this.refresh();
                    }, scope: this
                }
            }
        });        

        MODx.Ajax.request({
            url: this.config.url,
            params: {
                action: 'mgr/form/getform',           
            },
            listeners: {  
                success: {
                    fn: function (r) {     
                     
                        w.reset();
                        w.setValues(r.results);
                        w.show(e.target);
                        //this.refresh();
                    }, scope: this
                }
            }
        });
        
    },
   

    updateItem: function () {

        var row = this.getSelectionModel().getSelected();   

        MODx.Ajax.request({
            url: this.config.url,
            params: {
                action: 'mgr/item/update',
                uuid : Ext.util.JSON.encode(row.data.uuid)              
            },
            listeners: {
                success: {
                    fn: function () {
                        this.refresh();
                    }, scope: this
                }
            }
        })
    },

    updateaddItem: function () {

        var row = this.getSelectionModel().getSelected();   

        MODx.Ajax.request({
            url: this.config.url,
            params: {
                action: 'mgr/item/updateadd',
                id : Ext.util.JSON.encode(row.data.id),
                order_id : Ext.util.JSON.encode(row.data.order_id),
                operation : Ext.util.JSON.encode(row.data.type)              
            },
            listeners: {
                success: {
                    fn: function (r) {
                        this.refresh();
                    }, scope: this
                }
            }
        })
    },

    showCheck: function(){
        var row = this.getSelectionModel().getSelected();
        var payload = row.data.payload; 
        var mess = '';
        for (var i in payload) {
            mess = mess + _('modschetmash_'+i) + ': ' + payload[i]+ '<br />';
        }

        MODx.msg.alert(_('modschetmash_show_check'),
        mess,
        function() {          
          },MODx);
    },

    showError: function(){
        var row = this.getSelectionModel().getSelected();
        var err = row.data.error; 
        var mess = '';
        for (var i in err) {
            mess = mess + i + ': ' + err[i]+ '<br />';
        }

        MODx.msg.alert(_('modschetmash_error_check'),
        mess,
        function() {          
          },MODx);
    },

    removeItem: function () {
        var ids = this._getSelectedIds();
        if (!ids.length) {
            return false;
        }
        MODx.msg.confirm({
            title: ids.length > 1
                ? _('modschetmash_items_remove')
                : _('modschetmash_item_remove'),
            text: ids.length > 1
                ? _('modschetmash_items_remove_confirm')
                : _('modschetmash_item_remove_confirm'),
            url: this.config.url,
            params: {
                action: 'mgr/item/remove',
                ids: Ext.util.JSON.encode(ids),
            },
            listeners: {
                success: {
                    fn: function () {
                        this.refresh();
                    }, scope: this
                }
            }
        });
        return true;
    },
   

    getFields: function () {     
       return ['id', 'status', 'uuid', 'order_id', 'data_create',  'data_print_end', 'sum', 'type', 'payload', 'link', 'error', 'actions'];
    },

    getColumns: function () {
        return [{
            header: _('modschetmash_item_id'),
            dataIndex: 'id',
            sortable: true,
            width: 40
        }, {
            header: _('modschetmash_item_status'),
            dataIndex: 'status',
            sortable: true,
            renderer: modSchetmash.utils.renderLanguage,
            width: 80,
        }, {
            header: _('modschetmash_item_uuid'),
            dataIndex: 'uuid',
            sortable: true,
            width: 50,
        }, {
            header: _('modschetmash_item_order_id'),
            dataIndex: 'order_id',
            sortable: true,
            renderer: modSchetmash.utils.renderLinkOrder,
            width: 50,
        }, {
            header: _('modschetmash_item_data_create'),
            dataIndex: 'data_create',
            sortable: true,
            width: 100,
        },
        {
            header: _('modschetmash_item_data_print_end'),
            dataIndex: 'data_print_end',
            sortable: true,
            width: 100,
        },
        {
            header: _('modschetmash_item_sum'),
            dataIndex: 'sum',
            sortable: true,
            width: 100,
        },
        {
            header: _('modschetmash_item_type'),
            dataIndex: 'type',
            renderer: modSchetmash.utils.renderLanguage,
            sortable: true,
            width: 100,
        },
        {
            header: _('modschetmash_item_link_ofd'),
            dataIndex: 'link',   
            sortable: false,
            width: 80,         
        },
         {
            header: _('modschetmash_grid_actions'),
            dataIndex: 'actions',
            renderer: modSchetmash.utils.renderActions,
            sortable: false,
            width: 100,
            id: 'actions'
        }];
    },

    getTopBar: function () {
        return [{
            text: '<i class="icon icon-gear"></i>&nbsp;' + _('modschetmash_settings'),
            handler: this.createItem,
            scope: this
        }
        /*, '->', {
            xtype: 'modschetmash-field-search',
            width: 250,
            listeners: {
                search: {
                    fn: function (field) {
                        this._doSearch(field);
                    }, scope: this
                },
                clear: {
                    fn: function (field) {
                        field.setValue('');
                        this._clearSearch();
                    }, scope: this
                },
            }
        }*/
        ];
    },

    onClick: function (e) {
        var elem = e.getTarget();
        if (elem.nodeName == 'BUTTON') {
            var row = this.getSelectionModel().getSelected();
            if (typeof(row) != 'undefined') {
                var action = elem.getAttribute('action');
                if (action == 'showMenu') {
                    var ri = this.getStore().find('id', row.id);
                    return this._showMenu(this, ri, e);
                }
                else if (typeof this[action] === 'function') {
                    this.menu.record = row.data;
                    return this[action](this, e);
                }
            }
        }
        return this.processEvent('click', e);
    },

    _getSelectedIds: function () {
        var ids = [];
        var selected = this.getSelectionModel().getSelections();

        for (var i in selected) {
            if (!selected.hasOwnProperty(i)) {
                continue;
            }
            ids.push(selected[i]['id']);
        }

        return ids;
    },
    /*
    _doSearch: function (tf) {
        this.getStore().baseParams.query = tf.getValue();
        this.getBottomToolbar().changePage(1);
    },

    _clearSearch: function () {
        this.getStore().baseParams.query = '';
        this.getBottomToolbar().changePage(1);
    },
    */
});
Ext.reg('modschetmash-grid-items', modSchetmash.grid.Items);



